@@ -3,119 +3,7 @@ package Beekeeper::Config;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
-
-=head1 NAME
- 
-Beekeeper::Config - Read configuration files
- 
-=head1 VERSION
- 
-Version 0.01
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-Beekeeper applications use two config files to define how clients, workers
-and brokers connect to each other.
-
-These files are searched for in ENV C<BEEKEEPER_CONFIG_DIR>, C<~/.config/beekeeper>
-and then C</etc/beekeeper>.
-
-=head3 pool.config.json
-
-This file defines all worker pools running on this host, specifying 
-which logical bus should be used and which services it will run.
-
-The file format is in relaxed JSON, which allows comments and trailings commas.
-
-Each entry define a worker pool. Required parameters are:
-
-C<pool-id>: arbitrary identifier for the worker pool
-
-C<bus-id>: identifier of logical bus used by worker processes
-
-C<workers>: a map of worker classes to arbitrary config hashes
-
-Example:
-
-  [
-      {
-          "pool-id"     : "myapp",
-          "bus-id"      : "backend",
-          "description" : "pool of MyApp workers",
-  
-          "workers" : {
-              "MyApp::Service::Foo::Worker" : { "workers_count" : 4 },
-              "MyApp::Service::Bar::Worker" : { "workers_count" : 2 },
-          },
-      },
-  ]
-
-=head3 bus.config.json
-
-This file defines all logical buses used by your application, specifying
-the conection parameters to the STOMP brokers that will service them.
-
-For development purposes is handy to use a single broker to hold all 
-logical buses and easily simulate a complex topology, but in production 
-enviroments brokers should be isolated from each other.
-
-The file format is in relaxed JSON, which allows comments and trailings commas.
-
-Each entry define a logical bus. Accepted parameters are:
-
-C<bus-id>: unique identifier of the logical bus (required)
-
-C<cluster>: identifier of the cluster of logical buses that this bus belongs to (if any)
-
-C<host>: hostname or IP address of the broker (required)
-
-C<port>: port of the broker (default is 61613)
-
-C<tls>: if set to true enables the use of TLS on broker connection
-
-C<user>: username used to connect to the broker (required)
-
-C<pass>: password used to connect to the broker (required)
-
-C<vhost>: virtual host on broker (ignored by some brokers)
-
-Example:
-
-  [
-      {
-          "bus-id"  : "backend",
-          "host"    : "localhost",
-          "user"    : "backend",
-          "pass"    : "def456",
-          "vhost"   : "/back",
-      },
-      {
-          "bus-id"  : "frontend",
-          "host"    : "localhost",
-          "user"    : "frontend",
-          "pass"    : "def456",
-          "vhost"   : "/front",
-      },
-  ]
-
-=head1 METHODS
-
-=head3 get_bus_config( bus_id => $id )
-
-Reads and parse C<bus.config.json> and returns the config of the requested bus.
-
-=head3 get_pool_config( bus_id => $id )
-
-Reads and parse C<pool.config.json> and returns the config of the requested pool.
-
-=head3 read_config_file( $filename )
-
-Reads the given file and returns its content parsed as JSON.
-
-=cut
+our $VERSION = '0.03';
 
 use JSON::XS;
 
@@ -232,15 +120,125 @@ sub read_config_file {
 
 1;
 
+__END__
+
+=pod
+
 =encoding utf8
 
+=head1 NAME
+ 
+Beekeeper::Config - Read configuration files
+ 
+=head1 VERSION
+ 
+Version 0.03
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+Beekeeper applications use two config files to define how clients, workers
+and brokers connect to each other.
+
+These files are searched for in ENV C<BEEKEEPER_CONFIG_DIR>, C<~/.config/beekeeper>
+and then C</etc/beekeeper>.
+
+=head3 pool.config.json
+
+This file defines all worker pools running on this host, specifying 
+which logical bus should be used and which services it will run.
+
+The file format is in relaxed JSON, which allows comments and trailings commas.
+
+Each entry define a worker pool. Required parameters are:
+
+C<pool-id>: arbitrary identifier for the worker pool
+
+C<bus-id>: identifier of logical bus used by worker processes
+
+C<workers>: a map of worker classes to arbitrary config hashes
+
+Example:
+
+  [
+      {
+          "pool-id"     : "myapp",
+          "bus-id"      : "backend",
+          "description" : "pool of MyApp workers",
+  
+          "workers" : {
+              "MyApp::Service::Foo::Worker" : { "workers_count" : 4 },
+              "MyApp::Service::Bar::Worker" : { "workers_count" : 2 },
+          },
+      },
+  ]
+
+=head3 bus.config.json
+
+This file defines all logical buses used by your application, specifying
+the conection parameters to the MQTT brokers that will service them.
+
+For development purposes is handy to use a single broker to hold all 
+logical buses and easily simulate a complex topology, but in production 
+enviroments brokers should be isolated from each other.
+
+The file format is in relaxed JSON, which allows comments and trailings commas.
+
+Each entry define a logical bus. Accepted parameters are:
+
+C<bus-id>: unique identifier of the logical bus (required)
+
+C<cluster>: identifier of the cluster of logical buses that this bus belongs to (if any)
+
+C<host>: hostname or IP address of the broker
+
+C<port>: port of the broker (default is 1883)
+
+C<tls>: if set to true enables the use of TLS on broker connection
+
+C<username>: username used to connect to the broker
+
+C<password>: password used to connect to the broker
+
+Example:
+
+  [
+      {
+          "bus-id"   : "backend",
+          "host"     : "localhost",
+          "username" : "backend",
+          "password" : "def456",
+      },
+      {
+          "bus-id"   : "frontend",
+          "host"     : "localhost",
+          "username" : "frontend",
+          "password" : "def456",
+      },
+  ]
+
+=head1 METHODS
+
+=head3 get_bus_config( bus_id => $id )
+
+Reads and parse C<bus.config.json> and returns the config of the requested bus.
+
+=head3 get_pool_config( bus_id => $id )
+
+Reads and parse C<pool.config.json> and returns the config of the requested pool.
+
+=head3 read_config_file( $filename )
+
+Reads the given file and returns its content parsed as JSON.
+
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 José Micó.
+Copyright 2015-2021 José Micó.
 
 This is free software; you can redistribute it and/or modify it under the same 
 terms as the Perl 5 programming language itself.
