@@ -3,7 +3,32 @@ package Beekeeper::JSONRPC::Response;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
+
+
+sub new {
+    my $class = shift;
+
+    bless {
+        jsonrpc => '2.0',
+        result  => undef,
+        id      => undef,
+        @_
+    }, $class;
+}
+
+sub result  { $_[0]->{result} }
+sub id      { $_[0]->{id}     }
+
+sub success { 1 }
+
+1;
+
+__END__
+
+=pod
+
+=encoding utf8
 
 =head1 NAME
  
@@ -11,7 +36,7 @@ Beekeeper::JSONRPC::Response - Representation of a JSON-RPC response.
  
 =head1 VERSION
  
-Version 0.01
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -57,35 +82,13 @@ or not ($response->result cannot be trusted as it may be undefined on success).
 
 =back
 
-=cut
-
-sub new {
-    my $class = shift;
-
-    bless {
-        jsonrpc => '2.0',
-        result  => undef,
-        id      => undef,
-        @_
-    }, $class;
-}
-
-sub result  { $_[0]->{result} }
-sub id      { $_[0]->{id}     }
-
-sub success { 1 }
-
-1;
-
-=encoding utf8
-
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 José Micó.
+Copyright 2015-2021 José Micó.
 
 This is free software; you can redistribute it and/or modify it under the same 
 terms as the Perl 5 programming language itself.
