@@ -3,7 +3,32 @@ package Beekeeper::Service::LogTail;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
+
+
+use Beekeeper::Client;
+
+sub tail {
+    my ($class, %filters) = @_;
+
+    my $client = Beekeeper::Client->instance;
+
+    my $resp = $client->do_job(
+        method => '_bkpr.logtail.tail',
+        __auth => 'BKPR_ADMIN',
+        params => \%filters,
+    );
+
+    return $resp->result;
+}
+
+1;
+
+__END__
+
+=pod
+
+=encoding utf8
 
 =head1 NAME
 
@@ -11,7 +36,7 @@ Beekeeper::Service::LogTail - Buffer log entries
 
 =head1 VERSION
 
-Version 0.01
+Version 0.03
 
 =head1 SYNOPSIS
 
@@ -59,40 +84,17 @@ C<message>: Regex that applies to error messages.
 
 C<after>: Return only entries generated after given timestamp.
 
-=cut
-
-
-use Beekeeper::Client;
-
-sub tail {
-    my ($class, %filters) = @_;
-
-    my $client = Beekeeper::Client->instance;
-
-    my $resp = $client->do_job(
-        method => '_bkpr.logtail.tail',
-        __auth => 'BKPR_ADMIN',
-        params => \%filters,
-    );
-
-    return $resp->result;
-}
-
-1;
-
 =head1 SEE ALSO
  
 L<bkpr-log>.
 
-=encoding utf8
-
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 José Micó.
+Copyright 2015-2021 José Micó.
 
 This is free software; you can redistribute it and/or modify it under the same 
 terms as the Perl 5 programming language itself.
