@@ -3,34 +3,9 @@ package Beekeeper::Service::LogTail::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
-
-=head1 NAME
-
-Beekeeper::Service::LogTail::Worker - Buffer log entries
-
-=head1 VERSION
-
-Version 0.01
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-By default all workers use a C<Beekeeper::Logger> logger which logs errors and
-warnings both to files and to a topic C</topic/log> on the message bus.
-
-This worker keeps an in memory buffer of every log entry sent to that topic in
-every broker in a logical message bus.
-
-=head1 METHODS
-
-=head3 tail ( %filters )
-
-Returns all buffered entries that match the filter criteria.
-
-=cut
+our $VERSION = '0.03';
 
+use AnyEvent::Impl::Perl;
 use Beekeeper::Worker ':log';
 use base 'Beekeeper::Worker';
 
@@ -79,7 +54,7 @@ sub _connect_to_all_brokers {
             next;
         }
 
-        my $bus; $bus = Beekeeper::Bus::STOMP->new( 
+        my $bus; $bus = Beekeeper::MQTT->new( 
             %$config,
             bus_id     => $bus_id,
             timeout    => 300,
@@ -109,11 +84,11 @@ sub _collect_log {
     my ($self, $bus) = @_;
     weaken($self);
 
-    # Default logger logs to topics /topic/log.$level.$service
+    # Default logger logs to topic log/$level/$service
 
     $bus->subscribe(
-        destination    => "/topic/log.#",
-        on_receive_msg => sub {
+        topic      => "log/#",
+        on_publish => sub {
             my ($body_ref, $msg_headers) = @_;
 
             my $req = decode_json($$body_ref);
@@ -135,7 +110,7 @@ sub on_shutdown {
      foreach my $bus (@{$self->{_CLUSTER}}) {
 
         next unless ($bus->{is_connected});
-        $bus->disconnect( blocking => 1 );
+        $bus->disconnect;
     }
 }
 
@@ -190,13 +165,43 @@ sub tail {
 
 1;
 
+__END__
+
+=pod
+
+=encoding utf8
+
+=head1 NAME
+
+Beekeeper::Service::LogTail::Worker - Buffer log entries
+
+=head1 VERSION
+
+Version 0.03
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+By default all workers use a C<Beekeeper::Logger> logger which logs errors and
+warnings both to files and to a topic C</topic/log> on the message bus.
+
+This worker keeps an in memory buffer of every log entry sent to that topic in
+every broker in a logical message bus.
+
+=head1 METHODS
+
+=head3 tail ( %filters )
+
+Returns all buffered entries that match the filter criteria.
+
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 José Micó.
+Copyright 2015-2021 José Micó.
 
 This is free software; you can redistribute it and/or modify it under the same 
 terms as the Perl 5 programming language itself.
