@@ -3,60 +3,7 @@ package Beekeeper::Worker;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
-
-=head1 NAME
-
-Beekeeper::Worker - Base class for creating services
-
-=head1 VERSION
-
-Version 0.01
-
-=head1 SYNOPSIS
-
-  package MyApp::Worker;
-  
-  use Beekeeper::Worker ':log';
-  use base 'Beekeeper::Worker';
-
-  sub on_startup {
-      my $self = shift;
-      
-      $self->accept_notifications(
-          'myapp.msg' => 'got_message',
-      );
-      
-      $self->accept_jobs(
-          'myapp.sum' => 'do_sum',
-      );
-
-      log_debug 'Ready';
-  }
-  
-  sub authorize_request {
-      my ($self, $req) = @_;
-  
-      return REQUEST_AUTHORIZED;
-  }
-
-  sub got_message {
-      my ($self, $params) = @_;
-      warn $params->{message};
-  }
-  
-  sub do_sum {
-      my ($self, $params) = @_;
-      return $params->[0] + $params->[1];
-  }
-
-=head1 DESCRIPTION
-
-Base class for creating services.
-
-=head1 METHODS
-
-=cut
+our $VERSION = '0.03';
 
 use Beekeeper::Client ':worker';
 use Beekeeper::Logger ':log_levels';
@@ -68,10 +15,11 @@ use Sys::Hostname;
 use Scalar::Util 'blessed';
 use Carp;
 
-#TODO: our @CARP_NOT = ('AnyEvent', 'Beekeeper::Bus::STOMP');
+#TODO: our @CARP_NOT = ('AnyEvent', 'Beekeeper::MQTT');
 
 use constant COMPILE_ERROR_EXIT_CODE => 99;
-use constant REPORT_STATUS_PERIOD    => 10;
+use constant REPORT_STATUS_PERIOD    => 5;
+use constant UNSUBSCRIBE_LINGER      => 2;
 use constant REQUEST_AUTHORIZED      => int(rand(90000000)+10000000);
 
 use Exporter 'import';
@@ -110,74 +58,6 @@ sub log_trace    (@) { $LogLevel >= LOG_TRACE  && $Logger->( LOG_TRACE,  @_ ) }
 our $JSON;
 
 
-=head1 CONSTRUCTOR
-
-C<Beekeeper::Worker> objects are created automatically by C<Beekeeper::WorkerPool>
-after spawning new processes.
-
-=head1 METHODS
-
-=head3 on_startup
-
-This method is executed on a fresh worker process immediately after it was spawned.
-The connection to the broker was just stablished and the logger was initialized.
-
-It is placeholder intended to be overrided in subclasses, which in their implementation
-perform startup tasks and declare which job methods and notifications will be handled.
-
-This is the place to initialize, for example, persistent database or cache connections.
-
-After this method returns the worker will wait for incoming events to handle.
-
-=head3 on_shutdown
-
-This method is executed just before a worker process is stopped.
-
-It can be overrided as needed, the default implementation does nothing.
-
-=head3 authorize_request( $req )
-
-This method MUST be overrided in your worker classes, as the default behavior is
-to deny the execution of any request.
-
-When a request is received this method is called before executing the corresponding
-callback, and it must return the exported constant C<REQUEST_AUTHORIZED> in order to
-authorize it. Returning any other value will result in the request being ignored. 
-
-This is the place to handle application authentication and authorization.
-
-=head3 log_handler
-
-By default, all workers use a C<Beekeeper::Logger> logger which logs errors and
-warnings to files and also to a topic on the message bus. The command line tool
-C<bkpr-log> allows to inspect in real time the logs of the entire system. 
-
-To replace this default log mechanism for another one of your choice, you must 
-override the class C<log_handler> method and make that return an object implementing
-a C<log> method.
-
-For convenience you can import the ':log' symbols and expose to your class the
-functions C<log_fatal>, C<log_alert>, C<log_critical>, C<log_error>, C<log_warn>, 
-C<log_warning>, C<log_notice>, C<log_info>, C<log_debug> and C<log_trace>.
-
-These will call the underlying C<log> method of your logger class, if the severity
-is equal or higher than C<$Beekeeper::Worker::LogLevel>, which is set to allow 
-warnings by default. You can increase the log level to include debug info with 
-the --debug option of C<bkpr>, or from class config in file pool.config.json.
-
-Using these functions makes very easy to switch logging backends at a later date.
-
-All warnings and errors generated by the execution of the worker code are
-logged, unless you specifically catch and ignore them.
-
-=head3 Client methods
-
-In order to make RPC calls to another services, methods C<send_notification>, 
-C<do_job>, C<do_async_job>, C<do_background_job> and C<wait_all_jobs> are 
-automatically imported from C<Beekeeper::Client>.
-
-=cut
-
 sub new {
     my ($class, %args) = @_;
 
@@ -338,49 +218,6 @@ sub authorize_request {
     return undef; # do NOT authorize
 }
 
-=head3 accept_notifications ( $method => $callback, ... )
-
-Make this worker start accepting specified notifications from message bus.
-
-C<$method> is a string with the format "{service_class}.{method}". A default
-or fallback handler can be specified using a wildcard as "{service_class}.*".
-
-C<$callback> is a method name or a coderef that will be called when a notification
-is received. When executed, the callback will receive two parameters C<$params> 
-(which contains the notification data itself) and C<$req> which is a
-C<Beekeeper::JSONRPC::Notification> object (usually redundant unless you need to
-inspect request headers).
-
-Notifications are not expected to return a value, any value returned from its
-callback is ignored.
-
-The callback is executed within an eval block, if it dies the error will be logged
-but otherwise the worker will continue running.
-
-Example:
-
-  package MyWorker;
-  use base 'Beekeeper::Worker';
-  
-  sub on_startup {
-      my $self = shift;
-      
-      $self->accept_notifications(
-          'foo.bar' => 'bar',       # call $self->bar for notifications 'foo.bar'
-          'foo.baz' => $coderef,    # call $self->$coderef for notifications 'foo.baz'
-          'foo.*'   => 'fallback',  # call $self->fallback for any other 'foo.*'
-      );
-  }  
-  
-  sub bar {
-       my ($self, $params, $req) = @_
-       
-       # $self is a MyWorker object
-       # $params is a ref to the notification data
-       # $req is a Beekeeper::JSONRPC::Notification object
-  }
-
-=cut
 
 sub accept_notifications {
     my ($self, %args) = @_;
@@ -403,11 +240,13 @@ sub accept_notifications {
 
         my $local_bus = $self->{_BUS}->{cluster};
 
+        my $topic = "msg/$local_bus/$service/$method";
+        $topic =~ tr|.*|/#|;
+
         $self->{_BUS}->subscribe(
-            destination    => "/topic/msg.$local_bus.$service.$method",
-            ack            => 'auto', # means none
-            on_receive_msg => sub {
-                # ($body_ref, $msg_headers) = @_;
+            topic      => $topic,
+            on_publish => sub {
+                # ($payload_ref, $properties) = @_;
 
                 # Enqueue notification
                 push @{$worker->{job_queue_high}}, [ @_ ];
@@ -417,6 +256,10 @@ sub accept_notifications {
                     $worker->{busy_since} = Time::HiRes::time;
                     AnyEvent::postpone { $self->__drain_task_queue };
                 }
+            },
+            on_suback => sub {
+                my ($success, $prop) = @_;
+                croak "Could not subscribe to $topic" unless $success;
             }
         );
     }
@@ -440,52 +283,6 @@ sub __get_cb_coderef {
     }
 }
 
-=head3 accept_jobs ( $method => $callback, ... )
-
-Make this worker start accepting specified RPC requests from message bus.
-
-C<$method> is a string with the format "{service_class}.{method}". A default
-or fallback handler can be specified using a wildcard as "{service_class}.*".
-
-C<$callback> is a method name or a coderef that will be called when a request
-is received. When executed, the callback will receive two parameters C<$params> 
-(which contains the notification data itself) and C<$req> which is a
-C<Beekeeper::JSONRPC::Request> object (usually redundant unless you need to
-inspect request headers).
-
-The value or data ref returned by the callback will be sent back to the caller
-as response.
-
-The callback is executed within an eval block, if it dies the error will be 
-logged but otherwise the worker will continue running, and the caller will 
-receive a generic error response.
-
-Example:
-
-  package MyWorker;
-  use base 'Beekeeper::Worker';
-  
-  sub on_startup {
-      my $self = shift;
-      
-      $self->accept_jobs(
-          'foo.inc' => 'inc',       # call $self->inc for requests to 'foo.inc'
-          'foo.baz' => $coderef,    # call $self->$coderef for requests to 'foo.baz'
-          'foo.*'   => 'fallback',  # call $self->fallback for any other 'foo.*'
-      );
-  }
-  
-  sub bar {
-       my ($self, $params, $req) = @_
-       
-       # $self is a MyWorker object
-       # $params is a ref to the parameters of the request
-       # $req is a Beekeeper::JSONRPC::Request object
-       
-       return $params->{number} + 1;
-  }
-
-=cut
 
 sub accept_jobs {
     my ($self, %args) = @_;
@@ -516,11 +313,13 @@ sub accept_jobs {
 
         my $local_bus = $self->{_BUS}->{cluster};
 
+        my $queue = "\$share/BKPR/req/$local_bus/$service";
+        $queue =~ tr|.*|/#|;
+
         $self->{_BUS}->subscribe(
-            destination     => "/queue/req.$local_bus.$service",
-            ack             => 'client', # manual ack
-           'prefetch-count' => '1',
-            on_receive_msg  => sub {
+            topic       => $queue,
+            maximum_qos => 1,
+            on_publish  => sub {
                 # ($body_ref, $msg_headers) = @_;
 
                 # Enqueue job
@@ -531,6 +330,10 @@ sub accept_jobs {
                     $worker->{busy_since} = Time::HiRes::time;
                     AnyEvent::postpone { $self->__drain_task_queue };
                 }
+            },
+            on_suback => sub {
+                my ($success, $prop) = @_;
+                croak "Could not subscribe to $queue" unless $success;
             }
         );
     }
@@ -705,40 +508,49 @@ sub __drain_task_queue {
                     $json = $JSON->encode( $response );
                 }
 
-                # Request is marked as received just before sending the response. So, if the
+                # Request is ack'ed as received just after sending the response. So, if the
                 # process is abruptly interrupted here, the broker will send the request to
                 # another worker and it will be executed twice! (acking the request just before 
                 # processing it may cause unprocessed requests or undelivered responses)
 
-                $self->{_BUS}->ack(
-                    'id'           => $msg_headers->{'ack'},          # STOMP 1.2
-                    'message-id'   => $msg_headers->{'message-id'},   # STOMP 1.1
-                    'subscription' => $msg_headers->{'subscription'}, # STOMP 1.1
-                    'buffer_id'    => "txn-$request_id",
+                $self->{_BUS}->publish(
+                    topic     => $msg_headers->{'response_topic'},
+                    fwd_reply => $msg_headers->{'fwd_reply'},
+                    payload   => \$json,
+                    buffer_id => 'response',
                 );
 
-                $self->{_BUS}->send(
-                    'destination'     => $msg_headers->{'reply-to'},
-                    'x-forward-reply' => $msg_headers->{'x-forward-reply'},
-                    'buffer_id'       => "txn-$request_id",
-                    'body'            => \$json,
-                );
+                if (exists $msg_headers->{'packet_id'}) {
 
-                $self->{_BUS}->flush_buffer( 'buffer_id' => "txn-$request_id", );
+                    $self->{_BUS}->puback(
+                        packet_id => $msg_headers->{'packet_id'},
+                        buffer_id => 'response',
+                    );
+                }
+                else {
+                    # Should not happen (clients must publish with QoS 1)
+                    log_warn "Request published with QoS 0 to " . $msg_headers->{'topic'};
+                }
+
+                $self->{_BUS}->flush_buffer( buffer_id => 'response' );
             }
             else {
 
                 # Background jobs doesn't expect responses
 
-                $self->{_BUS}->ack(
-                    'id'           => $msg_headers->{'ack'},
-                    'message-id'   => $msg_headers->{'message-id'},
-                    'subscription' => $msg_headers->{'subscription'},
+                $self->{_BUS}->puback(
+                    packet_id => $msg_headers->{'packet_id'},
                 );
             }
         }
 
-        redo DRAIN if (@{$worker->{job_queue_high}} || @{$worker->{job_queue_low}})
+        redo DRAIN if (@{$worker->{job_queue_high}} || @{$worker->{job_queue_low}});
+
+        # Execute tasks postponed until job queue is empty
+        if (exists $worker->{postponed}) {
+            $_->() foreach @{$worker->{postponed}};
+            delete $worker->{postponed};
+        }
     }
 
     $_TASK_QUEUE_DEPTH--;
@@ -750,13 +562,6 @@ sub __drain_task_queue {
     $worker->{queued_tasks} = 0;
 }
 
-=head3 stop_accepting_notifications ( $method, ... )
-
-Make this worker stop accepting specified notifications from message bus.
-
-C<$method> must be one of the strings used previously in C<accept_notifications>.
-
-=cut
 
 sub stop_accepting_notifications {
     my ($self, @methods) = @_;
@@ -770,42 +575,50 @@ sub stop_accepting_notifications {
 
         my ($service, $method) = ($1, $2);
 
-        unless (defined $self->{_WORKER}->{callbacks}->{"msg.$fq_meth"}) {
+        my $worker = $self->{_WORKER};
+
+        unless (defined $worker->{callbacks}->{"msg.$fq_meth"}) {
             carp "Not previously accepting notifications $fq_meth";
             next;
         }
 
         my $local_bus = $self->{_BUS}->{cluster};
 
+        my $topic = "msg/$local_bus/$service/$method";
+        $topic =~ tr|.*|/#|;
+
+        # Cannot remove callbacks right now, as new notifications could be in flight or be 
+        # already queued. We must wait for unsubscription completion, and then until the 
+        # notification queue is empty to ensure that all received ones were processed. And 
+        # even then wait a bit more, as some brokers may send messages *after* unsubscription.
+        my $postpone = sub {
+
+            $worker->{_timers}->{"unsub-$topic"} = AnyEvent->timer( 
+                after => UNSUBSCRIBE_LINGER, cb => sub {
+
+                    delete $worker->{callbacks}->{"msg.$fq_meth"};
+                    delete $worker->{_timers}->{"unsub-$topic"};
+                }
+            );
+        };
+
         $self->{_BUS}->unsubscribe(
-            destination => "/topic/msg.$local_bus.$service.$method",
-            on_success  => sub {
+            topic       => $topic,
+            on_unsuback => sub {
+                my ($success, $prop) = @_;
 
-                delete $self->{_WORKER}->{callbacks}->{"msg.$fq_meth"};
+                #TODO: Report caller of stop_accepting_notifications method
+                warn "Could not unsubscribe from $topic" unless $success; 
 
-                # Discard notifications already queued
-                my $job_queue = $self->{_WORKER}->{job_queue_high};
+                my $postponed = $worker->{postponed} ||= [];
+                push @$postponed, $postpone;
 
-                @$job_queue = grep {
-                    my $task = $_;
-                    my ($body_ref, $msg_headers) = @$task;
-                    my $request = decode_json($$body_ref);
-                    my $req_method = $request->{method};
-                    $req_method =~ m/^([\.\w-]+)\.([\w-]+)$/;
-                    not ($service eq $1 && ($method eq '*' || $method eq $2));
-                } @$job_queue;
+                AnyEvent::postpone { $self->__drain_task_queue };
             }
         );
     }
 }
 
-=head3 stop_accepting_jobs ( $method, ... )
-
-Make this worker stop accepting specified RPC requests from message bus.
-
-C<$method> must be one of the strings used previously in C<accept_jobs>.
-
-=cut
 
 sub stop_accepting_jobs {
     my ($self, @methods) = @_;
@@ -820,46 +633,66 @@ sub stop_accepting_jobs {
         my ($service, $method) = ($1, $2);
 
         unless ($method eq '*') {
-            #TODO: Known limitation
+            # Known limitation. As all calls for an entire service class are received
+            # through a single MQTT subscription (in order to load balance them), it is 
+            # not possible to reject a single method. A workaround is to use a different
+            # class for each method that need to be individually rejected.
             croak "Cannot cancel individual job subscription to $fq_meth";
         }
 
-        my $callbacks = $self->{_WORKER}->{callbacks};
+        my $worker    = $self->{_WORKER};
+        my $callbacks = $worker->{callbacks};
 
         my @cb_keys = grep { $_ =~ m/^req.\Q$service\E\b/ } keys %$callbacks;
 
         unless (@cb_keys) {
+            #TODO: BUG: carp reports caller as Beekeeper/WorkerPool.pm line 440
             carp "Not previously accepting jobs $fq_meth";
             next;
         }
 
         my $local_bus = $self->{_BUS}->{cluster};
 
-        $self->{_BUS}->unsubscribe(
-            destination => "/queue/req.$local_bus.$service",
-            on_success  => sub {
+        my $topic = "\$share/BKPR/req/$local_bus/$service";
+        $topic =~ tr|.*|/#|;
 
-                delete $callbacks->{$_} foreach @cb_keys;
+        # Cannot remove callbacks right now, as new jobs could be in flight or be already 
+        # queued. We must wait for unsubscription completion, and then until the job queue 
+        # is empty to ensure that all received jobs were processed. And even then wait a
+        # bit more, as some brokers may send jobs *after* unsubscription.
+        my $postpone = sub {
 
-                my $job_queue = $self->{_WORKER}->{job_queue_low};
+            $worker->{_timers}->{"unsub-$topic"} = AnyEvent->timer( 
+                after => UNSUBSCRIBE_LINGER, cb => sub {
 
-                while (@$job_queue) {
+                    delete $worker->{callbacks}->{$_} foreach @cb_keys;
+                    delete $worker->{subscriptions}->{$service};
+                    delete $worker->{_timers}->{"unsub-$topic"};
 
-                    # NACK and discard jobs already queued
-                    my $task = shift @$job_queue;
-                    my ($body_ref, $msg_headers) = @$task;
-
-                    $self->{_BUS}->nack(
-                        'id'           => $msg_headers->{'ack'},          # STOMP 1.2
-                        'message-id'   => $msg_headers->{'message-id'},   # STOMP 1.1
-                        'subscription' => $msg_headers->{'subscription'}, # STOMP 1.1
-                    );
+                    # When shutting down tell _work_forever to stop
+                    $worker->{stop_cv}->end if $worker->{shutting_down};
                 }
+            );
+        };
+
+        $self->{_BUS}->unsubscribe(
+            topic        => $topic,
+            on_unsuback  => sub {
+                my ($success, $prop) = @_;
+
+                #TODO: Report caller of stop_accepting_jobs method
+                warn "Could not unsubscribe from $topic" unless $success; 
+
+                my $postponed = $worker->{postponed} ||= [];
+                push @$postponed, $postpone;
+
+                AnyEvent::postpone { $self->__drain_task_queue };
             }
         );
     }
 }
 
+
 sub __work_forever {
     my $self = shift;
 
@@ -885,30 +718,44 @@ sub __work_forever {
     }
 
     if ($self->{_BUS}->{is_connected}) {
-        $self->{_BUS}->disconnect( blocking => 1 );
+        $self->{_BUS}->disconnect;
     }
 }
 
-=head3 stop_working 
-
-Make this worker stop processing RPC requests and exit. Unprocessed jobs will 
-be resent to another worker by the message broker, unprocessed notifications
-will be ignored.
-
-This is the default signal handler for TERM signal. If this method is called
-manually WorkerPool will immediately respawn the worker again after it exits.
-
-=cut
 
 sub stop_working {
-    my $self = shift;
+    my ($self, %args) = @_;
 
-    unless ($self->{_WORKER}->{stop_cv}) {
-        # Worker was not fully initialized
+    my $worker = $self->{_WORKER};
+
+    # This is the default handler for TERM signal
+
+    unless (exists $worker->{stop_cv}) {
+        # Worker did not completed initialization yet
         CORE::exit(0);
     }
 
-    $self->{_WORKER}->{stop_cv}->send;
+    my %services;
+    foreach my $fq_meth (keys %{$worker->{callbacks}}) {
+        next unless $fq_meth =~ m/^req\.(?!_sync)(.*)\./;
+        $services{$1} = 1;
+    }
+
+    unless (keys %services) {
+        $worker->{stop_cv}->send;
+        return;
+    }
+
+    $worker->{shutting_down} = 1;
+
+    # Cannot exit right now, as some jobs could be in flight or already queued.
+    # So tell the broker to stop sending jobs, and exit after the job queue is empty
+    foreach my $service (keys %services) {
+
+        $worker->{stop_cv}->begin;
+
+        $self->stop_accepting_jobs( $service . '.*' );
+    }
 }
 
 
@@ -981,19 +828,248 @@ sub __report_exit {
 
 1;
 
+__END__
+
+=pod
+
+=encoding utf8
+
+=head1 NAME
+
+Beekeeper::Worker - Base class for creating services
+
+=head1 VERSION
+
+Version 0.03
+
+=head1 SYNOPSIS
+
+  package MyApp::Worker;
+  
+  use Beekeeper::Worker ':log';
+  use base 'Beekeeper::Worker';
+  
+  sub on_startup {
+      my $self = shift;
+      
+      $self->accept_notifications(
+          'myapp.msg' => 'got_message',
+      );
+      
+      $self->accept_jobs(
+          'myapp.sum' => 'do_sum',
+      );
+  
+      log_debug 'Ready';
+  }
+  
+  sub authorize_request {
+      my ($self, $req) = @_;
+  
+      return REQUEST_AUTHORIZED;
+  }
+  
+  sub got_message {
+      my ($self, $params) = @_;
+      warn $params->{message};
+  }
+  
+  sub do_sum {
+      my ($self, $params) = @_;
+      return $params->[0] + $params->[1];
+  }
+
+=head1 DESCRIPTION
+
+Base class for creating services.
+
+=head1 METHODS
+
+=head1 CONSTRUCTOR
+
+C<Beekeeper::Worker> objects are created automatically by C<Beekeeper::WorkerPool>
+after spawning new processes.
+
+=head1 METHODS
+
+=head3 on_startup
+
+This method is executed on a fresh worker process immediately after it was spawned.
+The connection to the broker was just stablished and the logger was initialized.
+
+It is placeholder intended to be overrided in subclasses, which in their implementation
+perform startup tasks and declare which job methods and notifications will be handled.
+
+This is the place to initialize, for example, persistent database or cache connections.
+
+After this method returns the worker will wait for incoming events to handle.
+
+=head3 on_shutdown
+
+This method is executed just before a worker process is stopped.
+
+It can be overrided as needed, the default implementation does nothing.
+
+=head3 authorize_request( $req )
+
+This method MUST be overrided in your worker classes, as the default behavior is
+to deny the execution of any request.
+
+When a request is received this method is called before executing the corresponding
+callback, and it must return the exported constant C<REQUEST_AUTHORIZED> in order to
+authorize it. Returning any other value will result in the request being ignored. 
+
+This is the place to handle application authentication and authorization.
+
+=head3 log_handler
+
+By default, all workers use a C<Beekeeper::Logger> logger which logs errors and
+warnings to files and also to a topic on the message bus. The command line tool
+C<bkpr-log> allows to inspect in real time the logs of the entire system. 
+
+To replace this default log mechanism for another one of your choice, you must 
+override the class C<log_handler> method and make that return an object implementing
+a C<log> method.
+
+For convenience you can import the ':log' symbols and expose to your class the
+functions C<log_fatal>, C<log_alert>, C<log_critical>, C<log_error>, C<log_warn>, 
+C<log_warning>, C<log_notice>, C<log_info>, C<log_debug> and C<log_trace>.
+
+These will call the underlying C<log> method of your logger class, if the severity
+is equal or higher than C<$Beekeeper::Worker::LogLevel>, which is set to allow 
+warnings by default. You can increase the log level to include debug info with 
+the --debug option of C<bkpr>, or from class config in file pool.config.json.
+
+Using these functions makes very easy to switch logging backends at a later date.
+
+All warnings and errors generated by the execution of the worker code are
+logged, unless you specifically catch and ignore them.
+
+=head3 Client methods
+
+In order to make RPC calls to another services, methods C<send_notification>, 
+C<do_job>, C<do_async_job>, C<do_background_job> and C<wait_all_jobs> are 
+automatically imported from C<Beekeeper::Client>.
+
+=head3 accept_notifications ( $method => $callback, ... )
+
+Make this worker start accepting specified notifications from message bus.
+
+C<$method> is a string with the format "{service_class}.{method}". A default
+or fallback handler can be specified using a wildcard as "{service_class}.*".
+
+C<$callback> is a method name or a coderef that will be called when a notification
+is received. When executed, the callback will receive two parameters C<$params> 
+(which contains the notification data itself) and C<$req> which is a
+C<Beekeeper::JSONRPC::Notification> object (usually redundant unless you need to
+inspect request headers).
+
+Notifications are not expected to return a value, any value returned from its
+callback is ignored.
+
+The callback is executed within an eval block, if it dies the error will be logged
+but otherwise the worker will continue running.
+
+Example:
+
+  package MyWorker;
+  use base 'Beekeeper::Worker';
+  
+  sub on_startup {
+      my $self = shift;
+      
+      $self->accept_notifications(
+          'foo.bar' => 'bar',       # call $self->bar for notifications 'foo.bar'
+          'foo.baz' => $coderef,    # call $self->$coderef for notifications 'foo.baz'
+          'foo.*'   => 'fallback',  # call $self->fallback for any other 'foo.*'
+      );
+  }  
+  
+  sub bar {
+       my ($self, $params, $req) = @_
+       
+       # $self is a MyWorker object
+       # $params is a ref to the notification data
+       # $req is a Beekeeper::JSONRPC::Notification object
+  }
+
+=head3 accept_jobs ( $method => $callback, ... )
+
+Make this worker start accepting specified RPC requests from message bus.
+
+C<$method> is a string with the format "{service_class}.{method}". A default
+or fallback handler can be specified using a wildcard as "{service_class}.*".
+
+C<$callback> is a method name or a coderef that will be called when a request
+is received. When executed, the callback will receive two parameters C<$params> 
+(which contains the notification data itself) and C<$req> which is a
+C<Beekeeper::JSONRPC::Request> object (usually redundant unless you need to
+inspect request headers).
+
+The value or data ref returned by the callback will be sent back to the caller
+as response.
+
+The callback is executed within an eval block, if it dies the error will be 
+logged but otherwise the worker will continue running, and the caller will 
+receive a generic error response.
+
+Example:
+
+  package MyWorker;
+  use base 'Beekeeper::Worker';
+  
+  sub on_startup {
+      my $self = shift;
+      
+      $self->accept_jobs(
+          'foo.inc' => 'inc',       # call $self->inc for requests to 'foo.inc'
+          'foo.baz' => $coderef,    # call $self->$coderef for requests to 'foo.baz'
+          'foo.*'   => 'fallback',  # call $self->fallback for any other 'foo.*'
+      );
+  }
+  
+  sub bar {
+       my ($self, $params, $req) = @_
+       
+       # $self is a MyWorker object
+       # $params is a ref to the parameters of the request
+       # $req is a Beekeeper::JSONRPC::Request object
+       
+       return $params->{number} + 1;
+  }
+
+=head3 stop_accepting_notifications ( $method, ... )
+
+Make this worker stop accepting specified notifications from message bus.
+
+C<$method> must be one of the strings used previously in C<accept_notifications>.
+
+=head3 stop_accepting_jobs ( $method, ... )
+
+Make this worker stop accepting specified RPC requests from message bus.
+
+C<$method> must be one of the strings used previously in C<accept_jobs>.
+
+=head3 stop_working 
+
+Make this worker stop processing RPC requests and exit. Unprocessed jobs will 
+be resent to another worker by the message broker, unprocessed notifications
+will be ignored.
+
+This is the default signal handler for TERM signal. If this method is called
+manually WorkerPool will immediately respawn the worker again after it exits.
+
 =head1 SEE ALSO
  
 L<Beekeeper::Client>, L<Beekeeper::WorkerPool>, L<Beekeeper::Logger>.
 
-=encoding utf8
-
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 José Micó.
+Copyright 2015-2021 José Micó.
 
 This is free software; you can redistribute it and/or modify it under the same 
 terms as the Perl 5 programming language itself.
