@@ -3,102 +3,7 @@ package Beekeeper::WorkerPool;
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
-
-=head1 NAME
-
-Beekeeper::WorkerPool - Manage worker pools
-
-=head1 VERSION
- 
-Version 0.01
-
-=head1 SYNOPSIS
-
-  $ bkpr --pool-id MyPool start
-  Starting pool of MyApp workers: beekeeper-MyPool.
-  
-  $ bkpr --pool-id MyPool stop
-  Stopping pool of MyApp workers: beekeeper-MyPool.
-  
-  $ bkpr --help
-  Usage: bkpr [options] {start|stop|restart|reload|check}
-   --foreground      Run in foreground (do not daemonize)
-   --pool-id    str  Worker pool name (mandatory)
-   --user       str  Run as specified user
-   --group      str  Run as specified group
-   --config-dir str  Path to directory containing config files
-   --debug           Turn on workers debug flag
-   --help            Shows this message
-
-=head1 DESCRIPTION
-
-This module contains the core of the command line tool C<bkpr> which is used
-to manage worker pools: it start, stop and monitor pools of persistent
-C<Beekeeper::Worker> processes which receive RPC requests from message bus.
-
-When started it daemonize itself (unless C<--foreground> option is passed) and
-fork all worker processes, then monitor those forked processes and B<immediately>
-respawn defunct ones.
-
-=head1 CONFIGURATION
-
-=head3 pool.config.json
-
-Workers pools are defined into a file named C<pool.config.json>, which is searched
-for into ENV C<BEEKEEPER_CONFIG_DIR>, C<~/.config/beekeeper> and C</etc/beekeeper>.
-The file is in relaxed JSON format (so it allows comments and trailing commas).
-
-All worker pools running on the host must be declared into the file, specifying 
-which logical bus should be used and which services it will run. 
-
-Each entry define a worker pool. Required parameters are:
-
-C<pool-id> An arbitrary identifier for the worker pool.
-
-C<bus-id> An identifier of logical bus used by worker processes.
-
-C<workers> A map of worker classes to (arbitrary) config hashes.
-
-The following example defines "MyApp" as a pool of 2 C<MyApp::Worker> processes:
-
-  [{
-      "pool-id" : "MyApp",
-      "bus-id"  : "backend",
-      "workers" : {
-          "MyApp::Worker" : { "workers_count" : 2 },
-      },
-  }]
-
-=head3 bus.config.json
-
-All logical buses used by your application are defined into a file named 
-C<bus.config.json> and specify the conection parameters to the STOMP brokers
-that will service them.
-
-Each entry define a logical bus. Required parameters are:
-
-C<bus-id> arbitrary identifier of logical bus.
-
-C<host> hostname or IP address of the STOMP broker.
-
-C<user> username used to connect to the STOMP broker.
-
-C<pass> password used to connect to the STOMP broker.
-
-C<vhost> virtual host of STOMP broker (may be omitted).
-
-The following example defines the logical bus "backend":
-
-  [{
-      "bus-id" : "backend",
-      "host"   : "10.0.0.1",
-      "user"   : "username",
-      "pass"   : "password",
-      "vhost"  : "/backend",
-  }]
-
-=cut
+our $VERSION = '0.03';
 
 use base 'Beekeeper::WorkerPool::Daemon';
 use POSIX ":sys_wait_h";
@@ -444,15 +349,117 @@ sub spawn_worker {
 
 1;
 
+__END__
+
+=pod
+
 =encoding utf8
 
+=head1 NAME
+
+Beekeeper::WorkerPool - Manage worker pools
+
+=head1 VERSION
+ 
+Version 0.03
+
+=head1 SYNOPSIS
+
+  $ bkpr --pool-id MyPool start
+  Starting pool of MyApp workers: beekeeper-MyPool.
+  
+  $ bkpr --pool-id MyPool stop
+  Stopping pool of MyApp workers: beekeeper-MyPool.
+  
+  $ bkpr --help
+  Usage: bkpr [options] {start|stop|restart|reload|check}
+   --foreground      Run in foreground (do not daemonize)
+   --pool-id    str  Worker pool name (mandatory)
+   --user       str  Run as specified user
+   --group      str  Run as specified group
+   --config-dir str  Path to directory containing config files
+   --debug           Turn on workers debug flag
+   --help            Shows this message
+
+=head1 DESCRIPTION
+
+This module contains the core of the command line tool C<bkpr> which is used
+to manage worker pools: it start, stop and monitor pools of persistent
+C<Beekeeper::Worker> processes which receive RPC requests from message bus.
+
+When started it daemonize itself (unless C<--foreground> option is passed) and
+fork all worker processes, then monitor those forked processes and B<immediately>
+respawn defunct ones.
+
+=head1 CONFIGURATION
+
+=head3 pool.config.json
+
+Workers pools are defined into a file named C<pool.config.json>, which is searched
+for into ENV C<BEEKEEPER_CONFIG_DIR>, C<~/.config/beekeeper> and C</etc/beekeeper>.
+The file is in relaxed JSON format (so it allows comments and trailing commas).
+
+All worker pools running on the host must be declared into the file, specifying 
+which logical bus should be used and which services it will run. 
+
+Each entry define a worker pool. Required parameters are:
+
+C<pool-id> An arbitrary identifier for the worker pool.
+
+C<bus-id> An identifier of logical bus used by worker processes.
+
+C<workers> A map of worker classes to (arbitrary) config hashes.
+
+The following example defines "MyApp" as a pool of 2 C<MyApp::Worker> processes:
+
+  [{
+      "pool-id" : "MyApp",
+      "bus-id"  : "backend",
+      "workers" : {
+          "MyApp::Worker" : { "workers_count" : 2 },
+      },
+  }]
+
+=head3 bus.config.json
+
+All logical buses used by your application are defined into a file named 
+C<bus.config.json> and specify the conection parameters to the MQTT brokers
+that will service them.
+
+Each entry define a logical bus. Required parameters are:
+
+C<bus-id>: unique identifier of the logical bus (required)
+
+C<cluster>: identifier of the cluster of logical buses that this bus belongs to (if any)
+
+C<host>: hostname or IP address of the broker
+
+C<port>: port of the broker (default is 1883)
+
+C<tls>: if set to true enables the use of TLS on broker connection
+
+C<username>: username used to connect to the broker
+
+C<password>: password used to connect to the broker
+
+
+The following example defines the logical bus "backend":
+
+  [{
+      "bus-id" : "backend",
+      "host"   : "10.0.0.1",
+      "user"   : "username",
+      "pass"   : "password",
+      "vhost"  : "/backend",
+  }]
+
 =head1 AUTHOR
 
 José Micó, C<jose.mico@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2015 José Micó.
+Copyright 2015-2021 José Micó.
 
 This is free software; you can redistribute it and/or modify it under the same 
 terms as the Perl 5 programming language itself.
