@@ -8,58 +8,53 @@ use base 'Tests::Service::Base';
 use Test::More;
 use Time::HiRes 'sleep';
 
-our $TIMEOUT = 2;
-
 
 sub start_test_workers : Test(startup => 1) {
     my $self = shift;
 
-    $TIMEOUT *= 10 if $self->automated_testing;
-
-    my $running = $self->start_workers('Tests::Service::Worker', workers_count => 11);
-    is( $running, 11, "Spawned 11 workers");
+    my $running = $self->start_workers('Tests::Service::Worker', workers_count => 3);
+    is( $running, 3, "Spawned 4 workers");
 };
 
-sub test_01_recursion : Test(11) {
+sub test_01_recursion : Test(3) {
 
     my $cli = Beekeeper::Client->instance;
     my $resp;
 
-    # Triangular number sequence
-    my @triangular = ( 0, 1, 3, 6, 10, 15, 21, 28, 36, 45, 55, 66, 78 ); 
+    my @factorial = ( 1, 1, 2, 6, 24, 120, 720, 5040, 40320, 362880, 3628800 ); 
 
     # No recursion
 
     $resp = $cli->do_job(
-        method  => 'test.triang',
-        params  => 1,
-        timeout => $TIMEOUT,
+        method  => 'test.fact',
+        params  => 2,
+        timeout => 10,
     );
 
-    is( $resp->result, 1, "triangular(1)");
+    is( $resp->result, 2, "factorial(2)");
 
     # 1 level of recursion
 
     $resp = $cli->do_job(
-        method  => 'test.triang',
-        params  => 2,
-        timeout => $TIMEOUT,
+        method  => 'test.fact',
+        params  => 3,
+        timeout => 10,
     );
 
-    is( $resp->result, 3, "triangular(2)");
+    is( $resp->result, 6, "factorial(3)");
 
-    for (my $i = 3; $i <= 11; $i++) {
+    # 2+ levels of recursion
+
+    for (my $i = 4; $i <= 4; $i++) {
 
         $resp = $cli->do_job(
-            method  => 'test.triang',
+            method  => 'test.fact',
             params  => $i,
-            timeout => $TIMEOUT,
+            timeout => 10,
         );
 
-        is( $resp->result, $triangular[$i], "triangular($i)");
+        is( $resp->result, $factorial[$i], "factorial($i)");
     }
-
-    # TODO: triangular(12) will timeout
 }
 
 sub test_02_recursion : Test(4) {
@@ -73,7 +68,7 @@ sub test_02_recursion : Test(4) {
     $resp = $cli->do_job(
         method  => 'test.fib1',
         params  => 1,
-        timeout => $TIMEOUT,
+        timeout => 10,
     );
 
     is( $resp->result, 1, "fib(1)");
@@ -81,7 +76,7 @@ sub test_02_recursion : Test(4) {
     $resp = $cli->do_job(
         method  => 'test.fib2',
         params  => 1,
-        timeout => $TIMEOUT,
+        timeout => 10,
     );
 
     is( $resp->result, 1, "fib(1)");
@@ -91,7 +86,7 @@ sub test_02_recursion : Test(4) {
     $resp = $cli->do_job(
         method  => 'test.fib1',
         params  => 2,
-        timeout => $TIMEOUT,
+        timeout => 10,
     );
 
     is( $resp->result, 1, "fib(2)");
@@ -99,36 +94,48 @@ sub test_02_recursion : Test(4) {
     $resp = $cli->do_job(
         method  => 'test.fib2',
         params  => 2,
-        timeout => $TIMEOUT,
+        timeout => 10,
     );
 
     is( $resp->result, 1, "fib(2)");
 }
 
-sub test_03_recursion : Test(4) {
+sub test_03_recursion : Test(5) {
     my $self = shift;
 
+    ## This test shows how effective is the broker doing load balance
+
+    if ($self->automated_testing) {
+        # It is hard to make this test run reliably on some smoke testers platforms
+        return "This test may fail when not enough system resources are available";
+    }
+
+    my $running = $self->start_workers('Tests::Service::Worker', workers_count => 8);
+    is( $running, 8, "Spawned 8 additional workers");
+
     my $cli = Beekeeper::Client->instance;
 
     my @fib = (0,1, 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144);
 
-    for (my $i = 3; $i <= 4; $i++) { # should handle up to fib(10) with proper load balance
+    #TODO: 11 workers should handle up to fib1(10) with proper load balance
+    for (my $i = 3; $i <= 4; $i++) {
 
         my $resp = $cli->do_job(
             method  => 'test.fib1',
             params  => $i,
-            timeout => $TIMEOUT,
+            timeout => 10,
         );
 
         is( $resp->result, $fib[$i], "fib($i)");
     }
 
-    for (my $i = 3; $i <= 4; $i++) { # should handle up to fib(5) with proper load balance
+    #TODO: 11 workers should handle up to fib2(5) with proper load balance
+    for (my $i = 3; $i <= 4; $i++) {
 
         my $resp = $cli->do_job(
             method  => 'test.fib2',
             params  => $i,
-            timeout => $TIMEOUT,
+            timeout => 10,
         );
 
         is( $resp->result, $fib[$i], "fib($i)");
@@ -143,18 +150,18 @@ sub test_04_client_api : Test(7) {
     my $svc = 'Tests::Service::Client';
     my $resp;
 
-    $resp = $svc->fibonacci_1( 4 );
+    $resp = $svc->fibonacci_1( 2 );
 
     isa_ok($resp, 'Beekeeper::JSONRPC::Response');
     is( $resp->success, 1 );
-    is( $resp->result, 3, "fib(4)");
+    is( $resp->result, 1, "fib(2)");
 
 
-    $resp = $svc->fibonacci_2( 4 );
+    $resp = $svc->fibonacci_2( 2 );
 
     isa_ok($resp, 'Beekeeper::JSONRPC::Response');
     is( $resp->success, 1 );
-    is( $resp->result, 3, "fib(4)");
+    is( $resp->result, 1, "fib(2)");
 }
 
 1;
